@@ -27,7 +27,7 @@ def pyprof2html_main():
                            '(default: html)')
     opts, args = parser.parse_args()
     if not len(args):
-        print parser.format_help()
+        print(parser.format_help())
         return 1
     p2h = Converter(args[0])
     p2h.functions_number = opts.print_functions
