@@ -7,10 +7,14 @@ import hashlib
 import os
 import re
 import time
-from hotshot import log, stats
+try:
+    from hotshot import log, stats
+except ImportError:
+    pass # TODO: xxx
 from pstats import Stats, func_std_string
 
 import pyprof2html
+from pyprof2html._compat import PY2
 from pyprof2html.environment import CODEC, ENVIRON
 
 __all__ = ['Converter']
@@ -18,20 +22,20 @@ __all__ = ['Converter']
 
 def check_hotlinetimings(dump):
     """check to linetimings hotshot-profile datafile."""
-    signature = "yes"
-    return signature in dump
+    signature = "yes".encode()
+    return True if dump.find(signature) > 0 else False
 
 
 def check_hotshot(dump):
     """check to hotshot-profile datafile."""
-    signature = "hotshot-version"
-    return signature in dump
+    signature = "hotshot-version".encode()
+    return True if dump.find(signature) > 0 else False
 
 
 def check_cprofile(dump):
     """check to cProfile module's data or profile module's data."""
-    signature = "sprof.Profiler"
-    return signature in dump
+    signature = "sprof.Profiler".encode()
+    return True if dump.find(signature) > 0 else False
 
 
 def mapping_table(target, nall):
@@ -97,14 +101,13 @@ def convert_unit(float_number):
 def create_stylefile(output_directory):
     """create to style.css and jquery.js file"""
     for filename in ('style.css', 'jquery.js', 'jquery.tablesorter.min.js'):
-        fp = open('%s/%s' % (output_directory, filename), 'w')
-        print >> fp, ENVIRON.get_template(filename).render().encode(CODEC)
-        fp.close()
+        with open('%s/%s' % (output_directory, filename), 'wb') as fp:
+            fp.write(ENVIRON.get_template(filename).render().encode(CODEC))
     filename = 'arrow.png'
     template_filename = 'arrow.png.base64'
-    fp = open('%s/%s' % (output_directory, filename), 'w')
-    fp.write(ENVIRON.get_template(template_filename).render().decode('base64'))
-
+    fp = open('%s/%s' % (output_directory, filename), 'wb')
+    fp.write(ENVIRON.get_template(template_filename).render().encode())
+    fp.close()
 
 class Converter(object):
     """output to HTML from profile data
@@ -116,10 +119,14 @@ class Converter(object):
         self.output_htmlfile = 'index.html'
         self.callers_htmlfile = 'callers.html'
         self.callers_data = []
-        dump = open(filename).read()
+        with open(filename, 'rb') as fp:
+            dump = fp.read()
         self.tmpl = ENVIRON.get_template('main.html')
         self.callers_tmpl = ENVIRON.get_template('callers.html')
         if check_hotshot(dump[:20]):
+            if not PY2:
+                # TODO: xxx
+                raise Exception('Not support for hotshot on Python3')
             self.prof = stats.load(filename)
             self.profile_datatype = "hotshot"
             if check_hotlinetimings(dump[102:108]):
@@ -127,7 +134,11 @@ class Converter(object):
                 self.tmplline = ENVIRON.get_template('hotshot-line.html')
                 self.profile_datatype = "hotshot(line)"
         else:
-            self.prof = Stats(filename)
+            try:
+                self.prof = Stats(filename)
+            except ValueError as e:
+                # TODO: xxx
+                raise ValueError(filename)
             if check_cprofile(dump):
                 self.profile_datatype = "cProfile"
             else:
@@ -169,7 +180,7 @@ class Converter(object):
                                    'totaltime': "%8.4lf" % self.prof.total_tt,
                                    'totalcalls': self.prof.total_calls}}
         if filecodec is None:
-            print >> outputfile, self.tmplline.render(renderdict)
+            outputfile.write(self.tmplline.render(renderdict))
         else:
             self.tmplline.stream(renderdict).dump(outputfile, CODEC)
 
@@ -239,7 +250,7 @@ class Converter(object):
 
         It refers to pstats.Stats.print_call_line() method.
         """
-        clist = call_dict.keys()
+        clist = list(call_dict.keys())
         clist.sort()
         tempdata = []
         for cnt, func in enumerate(clist):
@@ -259,7 +270,7 @@ class Converter(object):
                 dst_function = ""
             else:
                 dst_function = func_std_string(source)
-            callers_link = hashlib.md5(func_std_string(source)).hexdigest()
+            callers_link = hashlib.md5(func_std_string(source).encode()).hexdigest()
             tempdata.append({'dst_function': dst_function,
                              'callers_link': callers_link,
                              'ncalls': nc,
@@ -309,7 +320,7 @@ class Converter(object):
             cumtlevel = mapping_table(stat[3], self.prof.total_tt)
             cumclevel = mapping_table(float(cumpercall), self.prof.total_tt)
             linelink = "%s.html" % func[0].replace('/', '_')
-            callers_link = hashlib.md5(func_std_string(func)).hexdigest()
+            callers_link = hashlib.md5(func_std_string(func).encode()).hexdigest()
             data = {
                 'func': func,
                 'linelink': linelink,
@@ -338,7 +349,7 @@ class Converter(object):
         """
         if self.profile_datatype == 'hotshot(line)':
             profs = self._analyze_profline()
-            print self._print_sources(profs)
+            print(self._print_sources(profs))
         else:
             self.prof.sort_stats('time', 'calls')
             self.prof.print_stats()
@@ -356,15 +367,15 @@ class Converter(object):
         if self.profile_datatype == 'hotshot(line)':
             profs = self._analyze_profline()
             self._print_sources(profs)
-        print >> self.outfile, \
-                 self.tmpl.render(title=titletext,
-                                  proftime=self.proftime,
-                                  reporttime=self.reporttime,
-                                  profdata=profdata,
-                                  profiledata_count=self.profiledata_count,
+        self.outfile.write(
+            self.tmpl.render(title=titletext,
+                             proftime=self.proftime,
+                             reporttime=self.reporttime,
+                             profdata=profdata,
+                             profiledata_count=self.profiledata_count,
                                   thisname=self.output_htmlfile,
-                                  version=pyprof2html.__version__,
-                                  profile_datatype=self.profile_datatype)
+                             version=pyprof2html.__version__,
+                             profile_datatype=self.profile_datatype))
         self.outfile.close()
         filepath = "%s/%s" % (self.output_dir, self.callers_htmlfile)
         self.outfile = open(filepath, 'w')
@@ -377,8 +388,8 @@ class Converter(object):
                            profiledata_count=self.profiledata_count,
                            thisname=self.callers_htmlfile,
                            version=pyprof2html.__version__)
-        print >> self.outfile, \
-                 self.callers_tmpl.render(render_data)
+        self.outfile.write(
+            self.callers_tmpl.render(render_data))
         self.outfile.close()
 
     def printout(self, filetype='html', output_directory='html',
